/*
  $Id$
*/
#include "store.hpp"
#include <iostream>
#include <fstream>
#include <map>
#include <id3/tag.h>
#include <glib.h>
using namespace std;

static void parse_vorbis_tag(map<string, string>& tags, istream& is) {
    tags.clear();
    
    string line;
    while (is.good()) {
	getline(is, line, '\n');

	// drop CR
	if (line.size() > 0 && line[line.size()-1] == '\r') {
	    line.erase(line.size()-1);
	}

	// skip if this is not well formed
	string::size_type pos_eq = line.find('=');
	if (pos_eq == string::npos) {
	    continue;
	}
	tags[line.substr(0, pos_eq)] = line.substr(pos_eq+1);
    }
}

void store(const string& target, const string& infile) {
    istream* in;
    if (infile == "-") {
	in = &cin;
    }
    else {
	// REDFLAG: memory leak
	in = new ifstream(infile.c_str());
	if (!in->good()) {
	    cerr << "Failed to read " << infile << endl;
	    exit(1);
	}
    }

    map<string, string> tags;
    parse_vorbis_tag(tags, *in);

    ID3_Tag id3(target.c_str());
    id3.Strip(ID3TT_ALL);
    id3.Clear();
    for (map<string, string>::const_iterator ite = tags.begin();
	 ite != tags.end(); ite++) {
	ID3_Frame frame;
	if (ite->first == "TITLE") {
	    frame.SetID(ID3FID_TITLE);
	}
	else if (ite->first == "ALBUM") {
	    frame.SetID(ID3FID_ALBUM);
	}
	else if (ite->first == "ARTIST") {
	    frame.SetID(ID3FID_LEADARTIST);
	}
	else if (ite->first == "TRACKNUMBER") {
	    frame.SetID(ID3FID_TRACKNUM);
	}
	else {
	    cerr << "Unknown vorbis tag: " << ite->first << endl;
	    continue;
	}

	frame.GetField(ID3FN_TEXTENC)->Set(ID3TE_UNICODE);
	frame.GetField(ID3FN_TEXT)->SetEncoding(ID3TE_UNICODE);

	unicode_t* ustr = g_utf8_to_utf16(
	    ite->second.c_str(), -1, NULL, NULL, NULL);
	frame.GetField(ID3FN_TEXT)->Set((unicode_t*)ustr);
	g_free(ustr);
	
	id3.AddFrame(frame);
    }

    id3.SetPadding(false);
    id3.SetUnsync(false);
    id3.Update(ID3TT_ID3V2);
}
